"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const listr2_1 = require("listr2");
const axios_1 = (0, tslib_1.__importDefault)(require("axios"));
const fs = (0, tslib_1.__importStar)(require("fs-extra"));
const path = (0, tslib_1.__importStar)(require("path"));
const catalog_1 = (0, tslib_1.__importDefault)(require("../library/catalog"));
const txt_1 = (0, tslib_1.__importDefault)(require("../library/txt"));
const cover_1 = (0, tslib_1.__importDefault)(require("../library/cover"));
const epub_1 = (0, tslib_1.__importDefault)(require("../library/epub"));
const archiver_1 = (0, tslib_1.__importDefault)(require("../library/archiver"));
class Download extends core_1.Command {
    async run() {
        const reg = new RegExp('^[0-9]*$');
        let config;
        const { args: { bookId }, flags, } = await this.parse(Download);
        // 判断书籍 ID 合法性
        if (!reg.test(bookId)) {
            this.error('Your book ID is incorrect!');
        }
        // 构建全局配置项
        const hasConfig = fs.existsSync(path.join(this.config.configDir, 'config.json'));
        if (hasConfig) {
            config = fs.readJsonSync(path.join(this.config.configDir, 'config.json'));
        }
        else {
            config = {
                dir: path.join(this.config.home, '99books'),
                baseUrl: 'https://www.99csw.com',
                timeout: 5000,
                ua: 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36',
                thread: 1,
            };
            fs.createFileSync(path.join(this.config.configDir, 'config.json'));
            fs.writeJsonSync(path.join(this.config.configDir, 'config.json'), config);
        }
        globalThis.config = config;
        // 初始化下载文件夹
        if (!fs.existsSync(config.dir))
            fs.mkdirSync(config.dir);
        // 构建全局 axios 实例
        globalThis.ajax = axios_1.default.create({
            baseURL: config.baseUrl,
            timeout: config.timeout,
            headers: {
                'User-Agent': config.ua,
            },
        });
        // 指定下载任务
        const tasks = new listr2_1.Listr([
            {
                title: '获取书籍信息',
                task: async (ctx, task) => {
                    const book = await (0, catalog_1.default)(bookId, flags.cover);
                    if (book) {
                        ctx.book = book;
                        task.title = `《${book.name}》`;
                    }
                    else {
                        throw new Error('书籍信息获取失败');
                    }
                },
            },
            {
                title: '拉取章节',
                enabled: flags.type === 'txt',
                task: async (ctx, task) => {
                    task.title = `拉取章节\t0/${ctx.book.catalogs.length}`;
                    ctx.txt = await (0, txt_1.default)(ctx.book, task);
                    task.title = '章节获取成功';
                },
            },
            {
                title: '写出文件',
                enabled: flags.type === 'txt',
                task: async (ctx, task) => {
                    try {
                        const bookFilePath = path.join(config.dir, `${ctx.book.name}-${ctx.book.author}.txt`);
                        if (fs.existsSync(bookFilePath))
                            fs.removeSync(bookFilePath);
                        fs.createFileSync(bookFilePath);
                        fs.writeFileSync(bookFilePath, ctx.txt);
                        task.title = '写出成功';
                    }
                    catch (err) {
                        return new Error(err.msg);
                    }
                },
            },
            {
                title: '创建临时文件',
                enabled: flags.type === 'epub',
                task: async (ctx, task) => {
                    const tmpFilePath = path.join(config.dir, bookId);
                    if (fs.existsSync(tmpFilePath))
                        fs.removeSync(tmpFilePath);
                    fs.mkdirSync(tmpFilePath);
                    fs.createFileSync(path.join(tmpFilePath, 'mimetype'));
                    fs.writeFileSync(path.join(tmpFilePath, 'mimetype'), 'application/epub+zip');
                    fs.mkdirSync(path.join(tmpFilePath, 'META-INF'));
                    fs.mkdirSync(path.join(tmpFilePath, 'OPS'));
                    fs.mkdirSync(path.join(tmpFilePath, 'OPS', 'image'));
                    fs.mkdirSync(path.join(tmpFilePath, 'OPS', 'css'));
                    fs.mkdirSync(path.join(tmpFilePath, 'OPS', 'xhtml'));
                },
            },
            {
                title: '下载封面',
                enabled: flags.type === 'epub',
                task: async (ctx, task) => {
                    const res = await (0, cover_1.default)(ctx.book);
                    if (!res.success)
                        throw new Error(res.msg);
                },
            },
            {
                title: '写出文件',
                enabled: flags.type === 'epub',
                task: async (ctx, task) => {
                    task.title = `写出文件\t0/${ctx.book.catalogs.length}`;
                    await (0, epub_1.default)(ctx.book, task);
                    task.title = '写出成功';
                },
            },
            {
                title: '打包文件',
                enabled: flags.type === 'epub',
                task: async (ctx, task) => {
                    await (0, archiver_1.default)(ctx.book);
                },
            },
            {
                title: '清理工作目录',
                enabled: flags.type === 'epub',
                task: async (ctx, task) => {
                    const tmpFilePath = path.join(config.dir, bookId);
                    if (fs.existsSync(tmpFilePath))
                        fs.removeSync(tmpFilePath);
                },
            },
        ]);
        // 执行任务
        tasks.run();
    }
}
exports.default = Download;
Download.description = 'Download book with book id.';
Download.examples = ['<%= config.bin %> <%= command.id %>'];
Download.args = [
    {
        name: 'bookId',
    },
];
Download.flags = {
    // flag with a value (-n, --name=VALUE)
    cover: core_1.Flags.string({ char: 'c', description: 'use custom cover url' }),
    type: core_1.Flags.enum({
        char: 't',
        default: 'txt',
        options: ['txt', 'epub'],
    }),
};
